// Copyright Ryan Schmidt 2011.
// Distributed under the Boost Software License, Version 1.0.
// (See copy at http://www.boost.org/LICENSE_1_0.txt)


#include "ShapewaysIO.hxx"

#include <QtCore/QFile>
#include <QtCore/QByteArray>

#define qdebugbreak __debugbreak

// [RMS] this prints to visual studio window...
#define PRINT_DEBUG_STRING(x) qDebug("%s", x)



ShapewaysAPI::ShapewaysAPI()
    : m_http(this)
{
    m_bDebugPrint = false;

    m_username = "USERNAME";
    m_password = "PASSWORD";
    m_applicationID = "APPLICATION_ID";

    m_host = "www.shapeways.com";
    m_apiURI = "http://www.shapeways.com/modules/shapeways_api/webservice/v1/soap.php";
    m_apiPath = "/modules/shapeways_api/webservice/v1/soap.php";

    // hack qtsoap...
    qtsoap_set_XML_SCHEMA( "http://www.w3.org/2001/XMLSchema" );
    qtsoap_set_XML_SCHEMA_INSTANCE(	"http://www.w3.org/2001/XMLSchema-instance" );
    qtsoap_set_XML_NAMESPACE( "http://www.w3.org/XML/2001/namespace" );
    //qtsoap_set_ENVELOPE( "http://www.w3.org/2001/12/soap-envelope" );
    //qtsoap_set_ENCODING( "http://www.w3.org/2001/12/soap-encoding" );
    //qtsoap_set_soap_namespace( "soap" );

    QtSoapNamespaces::instance().registerNamespace("ns1", m_apiURI);
    // [RMS] not entirely sure about this...
    qtsoap_set_client_namespace( "ns1", "urn:SW.wsdl" );
}

ShapewaysAPI::~ShapewaysAPI()
{
}


void ShapewaysAPI::SetCredentials( const QString & username, const QString & password )
{
    m_username = username;
    m_password = password;
}


void ShapewaysAPI::Login()
{
    connect(&m_http, SIGNAL(responseReady(const QtSoapMessage &)),
            this, SLOT(loginResponse(const QtSoapMessage &)));

    QtSoapMessage request;
    request.setMethod( "login", m_apiURI );
    request.addMethodArgument( "username", "", m_username );
    request.addMethodArgument( "password", "", m_password );
    request.addMethodArgument( "application_id", "", m_applicationID );

    if ( m_bDebugPrint)
        qDebug() << request.toXmlString(2);

    m_http.setHost(m_host);
    m_http.setAction(m_apiURI);
    m_http.submitRequest(request, m_apiPath);
}


void ShapewaysAPI::loginResponse(const QtSoapMessage & message)
{
    if ( m_bDebugPrint ) {
        std::string xml(message.toXmlString(2).toAscii());
        PRINT_DEBUG_STRING("[ShapewaysAPI::loginResponse]\n");
        PRINT_DEBUG_STRING(xml.c_str());
    }

    if ( message.isFault() ) {
        QString tmp = qPrintable(message.faultString().toString());
        m_lastErrorMessage = QString("[ShapewaysAPI::loginResponse] ") + tmp;
        emit loginReturned(false);
    } else {
        QString result = message.returnValue().toString();
        m_sessionID = (const char *)result.toAscii();

        disconnect(&m_http, SIGNAL(responseReady(const QtSoapMessage &)),
                   this, SLOT(loginResponse(const QtSoapMessage &)));

        emit loginReturned(true);
    }
}




void ShapewaysAPI::GetPrinters()
{
    connect(&m_http, SIGNAL(responseReady(const QtSoapMessage &)),
            this, SLOT(getPrintersResponse(const QtSoapMessage &)));

    QtSoapMessage request;
    request.setMethod( "getPrinters", m_apiURI );
    request.addMethodArgument( "session_id", "", m_sessionID );
    request.addMethodArgument( "application_id", "", m_applicationID );

    m_http.setHost(m_host);
    m_http.setAction(m_apiURI);
    m_http.submitRequest(request, m_apiPath);
}


void ShapewaysAPI::getPrintersResponse(const QtSoapMessage & message)
{
    if ( m_bDebugPrint ) {
        qDebug("[ShapewaysAPI::getPrintersResponse]\n");
        qDebug() << message.toXmlString(2);
    }

    if ( message.isFault() ) {
        QString tmp = qPrintable(message.faultString().toString());
        m_lastErrorMessage = QString("[ShapewaysAPI::getPrintersResponse]") + tmp;
    } else {

        disconnect(&m_http, SIGNAL(responseReady(const QtSoapMessage &)),
                   this, SLOT(getPrintersResponse(const QtSoapMessage &)));
    }	
}



class QtSoapFloatType : public QtSoapType
{
public:
    QtSoapFloatType(const QtSoapQName &name, float fValue) : QtSoapType( name, QtSoapType::Float ) {
        v = fValue;
    }
    QtSoapFloatType &operator =(const QtSoapFloatType &copy) {
        t = copy.t;
        errorStr = copy.errorStr;
        i = copy.i;
        n = copy.n;
        u = copy.u;
        h = copy.h;
        v = copy.v;
        return *this;
    }
    ~QtSoapFloatType() {}

    void clear() { v.clear(); }

    bool parse(QDomNode node) { qdebugbreak(); return false; }
    bool isValid() const { return true; }

    QString toString() const { return v.toString(); }
    QVariant value() const { return v; }

    QDomElement toDomElement(QDomDocument doc) const {
        QString prefix = QtSoapNamespaces::instance().prefixFor(n.uri());
        QDomElement a = n.uri() == ""
                ? doc.createElement(n.name())
                : doc.createElementNS(n.uri(), prefix + ":" + n.name());
        QString schemaprefix = QtSoapNamespaces::instance().prefixFor(qtsoap_get_XML_SCHEMA_INSTANCE());
        a.setAttributeNS(qtsoap_get_XML_SCHEMA_INSTANCE(), schemaprefix + ":type", "xsd:" + typeName());
        a.appendChild(doc.createTextNode(v.toString()));
        return a;
    }

protected:
    QVariant v;
};



class QtSoapBase64Type : public QtSoapType
{
public:
    QtSoapBase64Type(const QtSoapQName &name, QString base64String) : QtSoapType( name, QtSoapType::Base64Binary ) {
        v = base64String;
    }
    QtSoapBase64Type &operator =(const QtSoapBase64Type &copy) {
        t = copy.t;
        errorStr = copy.errorStr;
        i = copy.i;
        n = copy.n;
        u = copy.u;
        h = copy.h;
        v = copy.v;
        return *this;
    }
    ~QtSoapBase64Type() {}

    void clear() { v.clear(); }

    bool parse(QDomNode node) { qdebugbreak(); return false; }
    bool isValid() const { return true; }

    QString toString() const { return v.toString(); }
    QVariant value() const { return v; }

    QDomElement toDomElement(QDomDocument doc) const {
        QString prefix = QtSoapNamespaces::instance().prefixFor(n.uri());
        QDomElement a = n.uri() == ""
                ? doc.createElement(n.name())
                : doc.createElementNS(n.uri(), prefix + ":" + n.name());
        QString schemaprefix = QtSoapNamespaces::instance().prefixFor(qtsoap_get_XML_SCHEMA_INSTANCE());
        a.setAttribute( schemaprefix + ":type", "xsd:" + typeName() );
        //a.setAttributeNS(qtsoap_get_XML_SCHEMA_INSTANCE(), schemaprefix + ":type", "xsd:" + typeName());
        a.appendChild(doc.createTextNode(v.toString()));
        return a;
    }

protected:
    QVariant v;
};


class SWModelType : public QtSoapStruct
{
public:
    SWModelType( const QtSoapQName &name ) : QtSoapStruct(name) {}

    QDomElement toDomElement(QDomDocument doc) const {
        QString prefix = QtSoapNamespaces::instance().prefixFor(n.uri());
        QString name = n.name();
        //QDomElement a = n.uri() == ""
        //		? doc.createElement(n.name())
        //		: doc.createElementNS(n.uri(), prefix + ":" + name);
        QDomElement a = doc.createElement("model");
        //a.setAttributeNS( qtsoap_get_XML_SCHEMA_INSTANCE(), "xsi:type", "typens:SWModel" );
        //a.setAttributeNS( qtsoap_get_XML_SCHEMA_INSTANCE(), "xsi:type", "typens:SWModel" );

        // [RMS] hardcoding to 'ns1' here, which I have manually added to qtsoap...
        a.setAttribute( "xsi:type", prefix + ":" + "SWModel" );

        SWModelType * pThis = const_cast<SWModelType *>(this);
        for (QtSoapStructIterator i(*const_cast<SWModelType *>(pThis)); i.data(); ++i)
            a.appendChild(i.data()->toDomElement(doc));
        return a;
    }
};


bool ShapewaysAPI::SubmitModel( const QString & filePath, ModelInfo info )
{
    // read file and convert to Base64
    QFile file(filePath);
    if ( ! file.open(QIODevice::ReadOnly) ) {
        m_lastErrorMessage.sprintf("[ShapewaysAPI::SubmitModel] cannot open file %s\n", filePath.toAscii());
        return false;
    }
    QFileInfo fi( file.fileName() );
    QString fileName = fi.fileName();

    file.seek(0);
    QByteArray byteArray = file.readAll();

    // [RMS] shapeways api expects double-encoding...
    //QByteArray base64File = byteArray.toBase64();
    QByteArray base64File = byteArray.toBase64().toBase64();

    // set up slot so that we know when upload is finished
    connect(&m_http, SIGNAL(responseReady(const QtSoapMessage &)),
            this, SLOT(submitModelResponse(const QtSoapMessage &)));

    // construct the request

    QtSoapMessage request;
    request.setMethod( "submitModel", m_apiURI );
    request.addMethodArgument( "session_id", "", m_sessionID );

    QtSoapStruct * pModel = new SWModelType(QtSoapQName("SWModel", m_apiURI));
    pModel->insert( new QtSoapSimpleType( QtSoapQName("desc"), info.description ) );
    pModel->insert( new QtSoapBase64Type( QtSoapQName("file"), base64File ) );
    pModel->insert( new QtSoapSimpleType( QtSoapQName("filename"), fileName ) );
    //pModel->insert( new QtSoapSimpleType( QtSoapQName("file_uri"), fileName ) );
    //pModel->insert( new QtSoapSimpleType( QtSoapQName("template_id"), 0 ) );
    pModel->insert( new QtSoapSimpleType( QtSoapQName("title"), info.title ) );

    QString orderable = (info.bOrderable) ? "1" : "0";
    pModel->insert( new QtSoapSimpleType( QtSoapQName("view_state"), orderable ) );
    pModel->insert( new QtSoapSimpleType( QtSoapQName("tags"), info.tags ) );
    pModel->insert( new QtSoapSimpleType(  QtSoapQName("has_color"), info.bHasColor, 0 ) );

    QString typestr = "";
    switch ( info.eModelType ) {
    case VRML: 	typestr = "VRML"; break;
    case STL:  	typestr = "STL"; break;
    case X3D:  	typestr = "X3D"; break;
    case X3DB:  typestr = "X3DB"; break;
    case X3DV:  typestr = "X3DV"; break;
    case DAE:  	typestr = "DAE"; break;
    case ZIP: 	typestr = "ZIP"; break;
    default:
        m_lastErrorMessage.sprintf( "[ShapewaysAPI::SubmitModel] unknown model type %d\n", info.eModelType );
        return false;
    }
    pModel->insert( new QtSoapSimpleType( QtSoapQName("modeltype"), typestr ) );

    //pModel->insert(  new QtSoapFloatType( QtSoapQName("scale"), 1.0f ) );
    //pModel->insert(  new QtSoapFloatType( QtSoapQName("markup"), 1.0f ) );
    request.addMethodArgument( pModel );

    //request.addMethodArgument( "tempateId", "", 0 );  //         <part name="templateId" type="xsd:integer" nillable="1"/>
    request.addMethodArgument( "application_id", "", m_applicationID );

    if ( m_bDebugPrint ) {
        qDebug() << request.toXmlString(2);
    }

    m_http.setHost(m_host);
    m_http.setAction(m_apiURI);
    m_http.submitRequest(request, m_apiPath);

    return true;
}


void ShapewaysAPI::submitModelResponse(const QtSoapMessage & message)
{
    if ( m_bDebugPrint ) {
        qDebug("[ShapewaysAPI::submitModelResponse]\n");
        qDebug() << message.toXmlString(2);

        emit submitModelReturned(false);
    }

    if ( message.isFault() ) {
        QString tmp = qPrintable(message.faultString().toString());
        m_lastErrorMessage = QString("[ShapewaysAPI::submitModelResponse]") + tmp;
    } else {
        QString result = message.returnValue().toString();

        disconnect(&m_http, SIGNAL(responseReady(const QtSoapMessage &)),
                   this, SLOT(submitModelResponse(const QtSoapMessage &)));

        emit submitModelReturned(true);
    }
}

