#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDebug>
#include <QFileDialog>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    connect(&m_api, SIGNAL(loginReturned(bool)), this, SLOT(loginResult(bool)));
    connect(&m_api, SIGNAL(submitModelReturned(bool)), this, SLOT(publishFileResult(bool)));
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::loginClicked()
{
    ui->loginResponseEdit->setText( "Trying..." );
    m_api.SetCredentials( ui->loginEdit->text(), ui->passwordEdit->text() );
    m_api.Login();
}
void MainWindow::loginResult(bool bResult)
{
    ui->loginResponseEdit->setText( (bResult) ? "OK" : "FAILED" );
}


void MainWindow::selectFileClicked()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open Mesh"), NULL,
            tr("DAE Files (*.dae)"));
    if ( ! fileName.isNull() )
            ui->filePathEdit->setText(fileName);
}


void MainWindow::publishFileClicked()
{
    QString fileName = ui->filePathEdit->text();
    if ( fileName.isNull() ) {
         ui->publishResultEdit->setText( "Please select a file first" );
        return;
    }
    if ( ! QFile::exists(fileName) ) {
       ui->publishResultEdit->setText( "File does not exist!" );
       return;
   }

    ui->publishResultEdit->setText( "Submitting..." );
    ShapewaysAPI::ModelInfo info;
    info.bHasColor = false;
    info.bOrderable = false;
    info.title = ui->titleEdit->text();
    info.description = ui->descriptionEdit->text();
    info.tags = ui->tagsEdit->text();
    info.eModelType = ShapewaysAPI::DAE;

    bool bOK = m_api.SubmitModel( fileName, info );
    if (! bOK )
        ui->publishResultEdit->setText( QString("Submission failed on client side! message is ") + m_api.GetLastError() );
}
void MainWindow::publishFileResult(bool bResult)
{
    ui->publishResultEdit->setText( (bResult) ? "OK" : "FAILED" );
}

